/* ------------------------------------------------------------------------
System       : cc65 cross-development environment
Platform     : Apple IIgs or Apple IIe with VOC or CB
Program      : fraglode (C)harlie+Bill 2014.
               All rights reserved.

Description  :

This is a Super Hi-Res (SHR) Image Loader for the Apple IIgs SHR
display (or equivalent) compiled in cc65. It is based on a loader that
Charlie and I did in Aztec C65 in 2013 and handles only the following
"raw" SHR file type:

File Type: $C1 (193)
Auxiliary Type: $CC65
Full Name: UnPacked Apple IIgs Super Hi-Res 16 Color Screen Image Fragment File
Short Name: UnPacked Super Hi-Res Image Fragment

Files of this type and auxiliary type contain an unpacked Apple IIGS Super
Hi-Res 16 color screen image fragment with or without Header, ColorTable and
Background Color.

Since this is a generic loader, the header and palette are both required.

This loader is really only a proof of concept with few comments and
limited error checking.

Bill Buckels
July 2014

Licence      : You may use this program for whatever you wish as long
			   as you agree that neither Charlie nor Bill have any
			   warranty or liability obligations whatsoever from said
			   use.
------------------------------------------------------------------------ */

#ifndef __APPLE2__
#define __APPLE2__
#endif

#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <dirent.h>

#include "shr.h"

/* load a raw SHR screen file */
int fraglode(char *name)
{

	FILE *fp;
	int i, c, status = -2;
	unsigned src1, src2, dest, width, height;

	fp = fopen(name,"rb");
    if (fp == NULL) return -1;
    /* clear the palette and scbs prior to loading */
	clearpalette();
	clearscbs();

    /* assume it's got a header */
    width = (unsigned)fgetc(fp);
    height = (unsigned) fgetc(fp);
    /* if not in range just bail */
    if (width > 160 || height > 200) {
		fclose(fp);
		return -3;
	}

    /* assume it's got a palette */
	c = fread((char *)&shrpal[0],1,32,fp);
    if (c != 32) {
		fclose(fp);
		return -4;
	}

    /* assume no background color - not supported for this loader */
    /* read image data */

    /* show full screen image after load */
    if (width == 160 && height == 200) {
		for (;;) {
			/* read image data */
			dest = 0x2000;	/* set destination to scanline 0 */
			for (i=0;i<4;i++) {
			   c = fread((char *)0x2000,1,8000,fp);
			   if (c != 8000)break;
			   maintoaux(0x2000,0x3f3f,dest);
			   dest += 8000;
			}
			if (c!=8000) break;
			status = 0;
			/* scb's already point to palette 1 */
			/* set palette 1 and show image */
			src1 = (unsigned) &shrpal[0];
			src2 = src1 + 31;
			maintoaux(src1,src2,(unsigned)0x9e00+32);
			break;
		}

		fclose(fp);
		return status;
	}

    /* show image fragment during load */
    /* scb's all point to blank palette 1 */
    /* point only the scb's we need to display the fragment
       to palette 0 - leave the other lines alone */
    /* set palette 0 */
    src1 = (unsigned) &shrpal[0];
    src2 = src1 + 31;
    maintoaux(src1,src2,0x9e00);

    /* clear the entire line buffer to black */
    memset(&scanline[0],0,160);
	for (;;) {
        /* center fragment */
        src1 = (200 - height)/2;
        src2 = src1 + height;
        dest =  (160 - width)/2;
 		for (i=src1;i<src2;i++) {
		   c = fread((char *)&scanline[dest],1,width,fp);
		   if (c != width)break;
		   putline(i);   /* put line */
		   /* point scb for this line to palette 0 */
		   setscb(i,0);  /* show line */
		}
		if (c!=width) break;
		status = 0;
		break;
	}
	fclose(fp);
	return status;

}

/* display up to 40 files in current directory */
int showpiclist(unsigned char d_type, unsigned d_auxtype)
{

    int cnt = 0;
    DIR *dir;
    struct dirent *dp;

    /* read files in current directory */
    if ((dir = opendir (".")) == NULL) return 0;

    while ((dp = readdir (dir)) != NULL) {
        if (cnt > 39) break;
        if (dp->d_type != d_type  || dp->d_auxtype != d_auxtype) continue;
        printf(" %-18s", dp->d_name);
        cnt++;
        if(cnt%2 == 0) puts("");
	}
	if(cnt%1 == 0) puts("");
    closedir(dir);
    return cnt;
}

int main(void)
{
	char buffer[66];
	int c;

    /* initialize text mode. stay in 40 column mode. */
    texton();
    for (;;) {
		clrscr();
        puts("$C1$CC65 SHRLoader (C)harlie+Bill 2014");
        puts("======================================");
        puts("Enter PIC Name or Blank to exit...");
        showpiclist(0xc1,0xcc65);
		printf("Name:");
		gets(buffer);
		if (buffer[0] == 0)break;

		/* turn shr on */
		shgron();

        /* load the image */
		c = fraglode(buffer);
		/* if nothing went wrong then wait for a keypress */
		if (c == 0) cgetc();

		/* clear the palette and scbs then
		   turn shroff and re-initialize text mode */
        clearpalette();
        clearscbs();
		shgroff();
		texton();

		/* if an error occurred, display the error and wait
		   for a keypress. otherwise skip this part */
		if (c != 0) {
			clrscr();
			switch(c) {
				case -1: printf("Can't open %s\n",buffer); break;
				case -2: puts("Error reading image data!");break;
				case -3: puts("Error reading header!");break;
				case -4: puts("Error reading palette!");break;
				default: printf("fraglode returned %d\n",c);

			}
			puts("Press a key...");
			cgetc();
		}

	}

	return 0;
}
